/**
 * Úplné minimum
 * @type Number
 */
minimum = 0;
/**
 * Úplné maximum (větší o 107)
 * @type Number
 */
maximum = 1107;

/**
 * Zjistí pozici elementu od levé strany obrazovky
 * @param {NodeList} element Element
 * @returns {Number} Pozice v pixelech
 */
function leftPosition(element) {
    var xPosition = 0;

    while(element) {
        xPosition += (element.offsetLeft - element.scrollLeft +
                element.clientLeft);
        element = element.offsetParent;
    }
    return xPosition;
}

/**
 * Získá druhou šipku
 * @param {boolean} side Zda-li se jedná o levou šipku či nikoliv
 * @param {int} number Číslo intervalu
 * @returns {NodeList|getSecondArrow.rightArrows|getSecondArrow.leftArrows}
 * Pravá nebo levá šipka
 */
function getSecondArrow(side, number) {
    if (side === true) {
        var rightArrows = document.getElementsByClassName("arrow-right");
        return rightArrows[number - 1];
    }
    else {
        var leftArrows = document.getElementsByClassName("arrow-left");
        return leftArrows[number - 1];
    }
}

/**
 * Vrátí určitou vybranou část v intervalu
 * @param {int} number Číslo intervalu
 * @returns {NodeList|getSelected.selectedElements} Vybraná část
 */
function getSelected(number) {
    var selectedElements = document.getElementsByClassName("selected");
    return selectedElements[number - 1];
}

/**
 * Upraví zadané šipce pozici od dané strany
 * @param {NodeList} e Element (šipka)
 * @param {string} side Strana
 * @param {int} number Číslo intervalu
 */
function update(e, side, number) {
    $(e).css(side, position - parseInt($(e).css("width")) / 2);
    
    // Upraví vybranou část
    updateSelected(e, number);
    
    // Šipka nesmí být za stranou intervalu
    if (parseInt($(e).css(side)) < 0)
        $(e).css(side, 0);
    // Šipka nesmí být úplně na pravé straně včetně šířky pravé strany
    if (parseInt($(e).css(side)) > parseInt($(".select").css("width")) -
            parseInt($(e).css("width")) * 2)
        $(e).css(side, parseInt($(".select").css("width")) -
            parseInt($(e).css("width")) * 2);
}

/**
 * Upraví šířku a strany vybrané části
 * @param {NodeList} e Element (šipka)
 * @param {int} number Číslo intervalu
 * @param {int} xPosition Pozice x myši (kvůli volání u upravení čísla)
 */
function updateSelected(e, number, xPosition) {    
    var selected = getSelected(number);
    // Pozice v intervalu v procentech
    var percents = ((xPosition) ? xPosition : position) / (parseInt($(".select").css("width")) / 100);
    // Zabránění vyjíždění vybrané části
    percents = (percents <= 5) ? 5 : percents;
    
    var secondArrow = getSecondArrow($(e).hasClass('arrow-left'), number);
    // Nastavení stran a definování šířky vybrané části
    var side = ($(e).hasClass('arrow-left')) ? "left" : "right";
    var secondSide = (side === "left") ? "right" : "left";
    var width;

    // Upravení strany šipky od kraje
    $(selected).css(side, percents + "%");
    // Definování minulého rozdílu šířky v procentech
    var difference = (100 * parseFloat($(secondArrow).css(secondSide)) /
            parseFloat($(secondArrow).parent().css("width")));
    // Výsledná šířka vybrané části
    width = 100 - difference - percents;
    
    $(selected).css("width", width + "%");
    
    // Výsledné číslo pro minimum/maximum
    var result;
    var x = Math.ceil((parseInt($(e).css("width")) + 5) * 4 / (parseInt($(".select").css("width")) / 100));
    
    // Zda-li se jedná o levou šipku
    if (side === "left") {
        result = percents * maximum / 100 + x;
        
        var minTexts = document.getElementsByClassName("min");
        var minText = minTexts[number - 1];
        $(minText).text(Math.ceil(result));
    }
    // Pravá šipka
    else {
        result = Math.abs(maximum * percents / 100 - maximum + x);
        
        var maxTexts = document.getElementsByClassName("max");
        var maxText = maxTexts[number - 1];
        $(maxText).text(Math.ceil(result));
    }
}

/**
 * Posouvání šipek
 * @param {NodeList} e Element (šipka)
 * @param {int} number Číslo intervalu
 */
function drag(e, number) {
    var secondArrow = getSecondArrow($(e).hasClass('arrow-left'), number);
    
    // Nastavení pozice do intervalu díky zrušení poloze intervalu od levé
    // strany monitoru
    var xPosition = event.clientX - leftPosition(e.parentElement);
    
    if ($(e).hasClass('arrow-left')) {
        // Ošetření, aby do sebe šipky nezajížděly
        if (parseInt($(e).css("left")) !== leftPosition(secondArrow) -
                leftPosition(e.parentElement) - parseInt($(e).css("width")) + 1)
            position = xPosition;
    }
    else {
        // Ošetření, aby do sebe šipky nezajížděly
        if (parseInt($(e).css("right")) !== parseInt($(".select").css("width")) -
                parseInt($(secondArrow).css("left")) - parseInt($(e).css("width") + 1)
                * 2) {
            // Přepočítání odsazení pravé šipky od pravé strany
            position = xPosition - parseInt($(".select").css("width"));
            position = Math.abs(position);
        }
    }
}

/**
 * Vykreslení pozice šipek
 * @param {NodeList} e Element (šipka)
 * @param {int} number Číslo intervalu
 */
function draw(e, number) {
    if ($(e).hasClass('arrow-left')) {
        update(e, "left", number);
    }
    else {
        update(e, "right", number);
    }
}

function disableDraggable(e, number) {
    var secondArrow = getSecondArrow($(e).hasClass('arrow-left'), number);
    // Zrušení pohyblivosti
    $(secondArrow).attr('draggable', 'false');
    $(secondArrow).attr('ondragstart', '');
    $(secondArrow).attr('ondrag', '');
    $(secondArrow).attr('ondragover', '');
}

/**
 * Opět nastaví pohyblivost s elementem
 * @param {NodeList} e Element
 * @param {int} number Číslo intervalu
 */
function setDraggable(e, number) {
    var secondArrow = getSecondArrow($(e).hasClass('arrow-left'), number);
    $(secondArrow).attr('draggable', 'true');
    $(secondArrow).attr('ondragstart', 'disableDraggable(this, ' + number + ')');
    $(secondArrow).attr('ondrag', 'drag(this, ' + number + ')');
    $(secondArrow).attr('ondragover', 'draw(this, ' + number + ')');
}

/**
 * Upraví číslo ve spanu
 * @param {NodeList} e Element (input)
 * @param {int} number Číslo intervalu
 * @param {int} value Číslo z inputu
 * @returns {Boolean} Vrací false při špatné hodnotě
 */
function updateNumber(e, number, value) {
    // Zda-li se jedná o input k minimální hodnotě či nikoliv
    var isMin = $(e).hasClass('input-min');
    var ownText = (isMin) ? 'min' : 'max';

    // Text inputu
    var itsTexts = document.getElementsByClassName(ownText);
    var itsText = itsTexts[number - 1];
    
    // Druhý text
    var secondInput = (isMin) ? 'max' : 'min';
    var secondTexts = document.getElementsByClassName(secondInput);
    var secondText = secondTexts[number - 1];
    
    // Hodnota musí být v rozmezí minima a maxima
    if (value < minimum || value > maximum - 107) {
        alert('Hodnota musí být v rozmezí ' + minimum + ' - ' + parseInt(maximum - 107) + '!');
        e.value = '';
        return false;
    }
    
    // Šipky se nesmí přestřelit pomocí hodnot
    if (Math.abs(value - (maximum - 107)) < $(secondText).text()) {
        alert('Tato hodnota je neplatná.');
        return false;
    }
    
    // Pokud se jedná o input s maximální hodnotou, nesmí být hodnota menší než
    // u minima
    if (!isMin) {
        if ($(secondText).text() > value && $(secondText).text() !== maximum - 107) {
            value = parseInt($(secondText).text()) + 1;
        }
    }
    // Pokud se jedná o input s minimální hodnotou, nesmí být hodnota větší než
    // u maxima
    else if (isMin && secondText.value < value && secondText.value) {
        value = parseInt(secondText.value) - 1;
    }
    
    // Levá šipka
    if (isMin) {
        // Výpočet odsazení zleva
        var fromLeft = value / (maximum - 107) * (parseInt($(".select").css("width")) -
                parseInt($(".arrow").css("width")) * 2);
        var leftArrows = document.getElementsByClassName("arrow-left");
        var leftArrow = leftArrows[number - 1];
        
        $(leftArrow).css("left", fromLeft);
        // Přidání půlky šířky šipky, aby vybraná část nečouhala
        updateSelected(leftArrow, number, fromLeft +
                parseInt($(".arrow").css("width")) / 2);
    }
    // Pravá šipka
    else {
        // Výpočet odsazení zprava
        var fromRight = Math.abs(Math.round(((value + (maximum - 107)) /
                (maximum - 107) * (parseInt($(".select").css("width")) -
                parseInt($(".arrow").css("width")) * 2) / 10000) -
                parseInt($(".select").css("width")) +
                parseInt($(".arrow").css("width")) * 2));
        var rightArrows = document.getElementsByClassName("arrow-right");
        var rightArrow = rightArrows[number - 1];
        
        $(rightArrow).css("right", fromRight);
        // Přidání celé šířky šipky, jinak to úplně čouhá
        updateSelected(rightArrow, number, fromRight +
                parseInt($(".arrow").css("width")));
    }

    $(itsText).text(value);
}